<?php
	Class User extends CI_Model {
		/*
		 *	Constructor
		 */
		function __construct() {
			parent::__construct();
			
			//Load required libraries and drivers
			$this->load->database();
			$this->load->library('session');
		}


		/*
		 *	This function returns the users password hint (or FALSE if user isn't logged in).
		 *  RESTRICTED-LEVEL: None
		 */ 
		public function getPasswordHint($user) {
			//Query-structure
			$this->db->select('passwdHint');
			$this->db->from('Users');
			$this->db->where('userName', $user);
			$this->db->limit(1);
			
			//Execute query
			$query = $this->db->get();
			$result = $query->result();
			
			//If a matching DB record is found.
			if($result) {
				foreach($result as $row) {
					$hint = $row->passwdHint;
					
					//Return hint
					return $hint;
				}
			}
			
			//No such user
			return FALSE;
		}


		/*
		 *	This function logs the user in (returns FALSE on fail).
		 *  RESTRICTED-LEVEL: None
		 */
		public function login($username, $password) {			
			//Generate a salted hash
			$hash = $this->getSaltedHash($password);
	
			//Query-structure
			$this->db->select('userName, name, passwd, userType, ssn'); // Tog bort firstLogin här.
			$this->db->from('Users');
			$this->db->where('userName', $username);
			$this->db->where('passwd', $hash);
			$this->db->limit(1);
			
			//Execute query
			$query = $this->db->get();
			$result = $query->result();
			
			//If a matching DB record is found.
			if($result) {
				//Prepare session data
				$userDetails = array();
				foreach($result as $row) {
					$userDetails = array(
						'username' => $row->userName,
						'name' => $row->name,
						'usertype' => $row->userType,
						'ssn' => $row->ssn,
					// 	'firstLogin' => $row->firstLogin
					);
				}
				
				//Set session data
				$this->session->set_userdata('authenticated', $userDetails);
				
				//Log attempt as valid
				$this->logLogin($username, 1);
				
				//Return success
				return TRUE;
			}

			//Log attempt as invalid
			$this->logLogin($username, 0);

			//Return fail
			return FALSE;
		}


		/*
		 *	This function logs the user out.
		 *  RESTRICTED-LEVEL: Self
		 */
		public function logout() {
			//Unset session data
			$this->session->unset_userdata('authenticated');
		}


		/*
		 *	This function changes the users password.
		 *  RESTRICTED-LEVEL: Self
		 */
		public function changePassword($pwdOld, $pwdNew, $pwdHint) {
			//Check that a user is logged in.
			if($this->isLoggedIn()) {
				$user = $this->getUserName();
				$oldHash = $this->getSaltedHash($pwdOld);
				$newHash = $this->getSaltedHash($pwdNew);
				
				//Validate input with database
				$this->db->select('userName');
				$this->db->from('Users');
				$this->db->where('userName', $user);
				$this->db->where('passwd', $oldHash);
				$this->db->limit(1);
				$query = $this->db->get();
				$result = $query->result();
				
				//If a matching DB record is found, update database.
				if($result) {
					$data = array(
						'passwd' => $newHash,
						'passwdHint' => $pwdHint
					);
					
					$this->db->where('userName', $user);
					$this->db->update('Users', $data);
					
					//Return Success!
					return TRUE;
				}
			}
			
			//Return error
			return FALSE;
		}


		/*
		 *	This function registers user into the database.
		 *  RESTRICTED-LEVEL: Teacher
		 */
		public function addUser($userName, $name, $ssn, $password, $userType, $pwdHint) {
			//Check that a user is logged in and has the right privileges (is teacher)
			if($this->isLoggedIn() && $this->getUserType() === 'Teacher') {
				//Generate a salted hash
				$hash = $this->getSaltedHash($password);
				
				//Query-structure (All values are escaped automatically by codeigninte, producing safer queries.)
				$this->db->set('userName', $userName);
				$this->db->set('name', $name);
				$this->db->set('ssn', $ssn);
				$this->db->set('passwd', $password);
				$this->db->set('userType', $userType);
				$this->db->set('passwdHint', $pwdHint);
				$result = $this->db->insert('Users');
				
				//Check for my-sql error
				if($result) {
					//Return success
					return TRUE;
				}
			}
			
			//Return error
			return FALSE;
		}


		/*
		 *	This function removes users from the database.
		 *  RESTRICTED-LEVEL: Teacher
		 */
		public function removeUser($userName) {
			//Check that a user is logged in, has the right privileges (is teacher) and not is the users own username.
			if($this->isLoggedIn() && $this->getUserType() === 'Teacher' && $this->getUserName() != $userName) {
				//Query-structure
				$this->db->where('userName', $userName);
				$result = $this->db->delete('Users');
				
				//Check for my-sql error
				if($result) {
					//Return success
					return TRUE;
				}
			}
			
			//Return error
			return FALSE;
		}


		/*
		 *	This reset the password for the user.
		 *  RESTRICTED-LEVEL: Teacher
		 */
		public function resetUser($userName) {
			//Check that a user is logged in, has the right privileges (is teacher) and not is the users own username.
			if($this->isLoggedIn() && $this->getUserType() === 'Teacher' && $this->getUserName() != $userName) {
				//Check user type
				$this->db->select('userName, userType, ssn, email');
				$this->db->from('Users');
				$this->db->where('userName', $username);
				$this->db->limit(1);
				$query = $this->db->get();
				$result = $query->result();
				
				//If a matching DB record is found.
				if($result) {
					//Prepare new hash depending on user-type
					$newPwdHash = '';
					
					if ($row->userType == 'Student') {
						$newPwdHash = $this->getSaltedHash($row->ssn);
					}
					else if ($row->userType == 'Teacher') {
						//$newPwdHash = $this->getSaltedHash($row->email);
						$newPwdHash = $this->getSaltedHash($row->email);
					}
					
					//Execute reset
					$data = array(
						'passwd' => $newPwdHash,
						'passwdHint' => 'default',
						'firstLogin' => 1
					);
					
					$this->db->where('userName', $userName);
					$this->db->update('Users', $data);
					
					//Return Success!
					return TRUE;
				}
			}
			
			//Return error
			return FALSE;
		}


		/*
		 *	This parser a user list from ladok.
		 *  RESTRICTED-LEVEL: Teacher
		 */
		public function parseLadok() {
			//Check that a user is logged in, has the right privileges (is teacher) and not is the users own username.
			if($this->isLoggedIn() && $this->getUserType() === 'Teacher' && $this->getUserName() != $userName) {
			}
		}


		/*
		 *	Generates a salted password hash, encrypted with sha1.
		 *  RESTRICTED-LEVEL: System
		 */
		private function getSaltedHash($pwd) {
			//Salt = CodeIgniters encryption-key from config
			$salt = $this->config->item('encryption_key');
			
			//Generate SHA1 hash using salt
			$hash = sha1($salt.$pwd);
			
			return $hash;
		}


		/*
		 *	Log the login attempt.
		 *  RESTRICTED-LEVEL: System
		 */
		private function logLogin($userName, $valid) {
			$data = array(
				'userName' => $userName,
				'userAgent' => $this->session->userdata('user_agent'),
				'userIP' => $this->session->userdata('ip_address'),
				'browserID' => $this->session->userdata('session_id'),
				'success' => $valid
			);
			
			$this->db->insert('logUserLoginAttempts', $data);
		}


		/*
		 *	This function return TRUE if the user is logged in and FALSE otherwise.
		 *  RESTRICTED-LEVEL: System
		 */
		public function isLoggedIn() {
			if ($this->session->userdata('authenticated')) {
				return TRUE;
			}
			else{
				return FALSE;
			}
		}


		/*
		 *	This function returns the users type (or FALSE if user isn't logged in).
		 *  RESTRICTED-LEVEL: System
		 */
		public function getUserType() {
			if($this->isLoggedIn()) {
				$temp = $this->session->userdata('authenticated');
				return $temp['usertype'];
			}
			
			return FALSE;
		}


		/*
		 *	This function returns a boolean containing information if it is the first login.
		 *  RESTRICTED-LEVEL: System
		 */
		public function isFirstLogin() {
			if($this->isLoggedIn()) {
				$temp = $this->session->userdata('authenticated');
				if ($temp['firstLogin'] == 1) {
					return TRUE;
				}
			}
			
			return FALSE;
		}


		/*
		 *	This function returns the username (or FALSE if user isn't logged in).
		 *  RESTRICTED-LEVEL: System
		 */ 
		public function getUserName() {
			if($this->isLoggedIn()) {
				$temp = $this->session->userdata('authenticated');
				return $temp['username'];
			}
			
			return FALSE;
		}
	}
	
?>